"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var sl_env_vars_1 = require("./constants/sl-env-vars");
var fs = require('fs');
var files_utils_1 = require("./utils/files-utils");
var ignore_1 = require("ignore");
var IgnoredFilesHandler = /** @class */ (function () {
    function IgnoredFilesHandler(slIgnoreFilePath, outputPath, filename, logger) {
        this.isUsingNycConfig = false;
        this.slIgnoreFilePath = slIgnoreFilePath;
        this.outputPath = outputPath;
        this.filename = filename;
        this.logger = logger;
        this.initIgnorePatterns();
    }
    IgnoredFilesHandler.prototype.initIgnorePatterns = function () {
        this.ignoredPatterns = ignore_1.default().add(this.getRawPatterns());
    };
    IgnoredFilesHandler.prototype.getRawPatterns = function () {
        if (!this.slIgnoreFilePath) {
            this.logger.info("'%s' file does not exists, trying to load nyc configuration.", this.filename);
            this.rawPattern = this.readFromNycConfig();
        }
        else {
            this.rawPattern = this.readFromSlIgnoreFile();
        }
        if (this.rawPattern) {
            return this.addExcludedPath(this.rawPattern, this.outputPath);
        }
        return this.getDefaultPattern();
    };
    IgnoredFilesHandler.prototype.readFromSlIgnoreFile = function () {
        var pattern;
        try {
            pattern = fs.readFileSync(this.slIgnoreFilePath, 'utf8');
            this.logger.info("Using '%s' file '%s'", this.filename, this.slIgnoreFilePath);
        }
        catch (err) {
            this.logger.info("Failed to read from: '%s'. Using default behavior.", this.slIgnoreFilePath);
        }
        return pattern;
    };
    IgnoredFilesHandler.prototype.getFormattedPattern = function () {
        if (!this.rawPattern) {
            return null;
        }
        return this.rawPattern.replace(/\n/g, "; ");
    };
    IgnoredFilesHandler.prototype.addExcludedPath = function (pattern, excludedFolder) {
        if (!excludedFolder)
            return pattern;
        if (excludedFolder && excludedFolder[excludedFolder.length - 1] != '/') { //Ensure the excludedFolder ends with / to comply with gitignore
            excludedFolder += '/';
        }
        pattern = pattern + "\n" + excludedFolder;
        return pattern;
    };
    IgnoredFilesHandler.prototype.getDefaultPattern = function () {
        // default pattern - all js files under test directory and it's sub directories.
        var pattern = "";
        pattern = this.addExcludedPath(pattern, this.outputPath);
        return pattern;
    };
    IgnoredFilesHandler.prototype.readFromNycConfig = function () {
        try {
            var cwd = sl_env_vars_1.SlEnvVars.getBasePath() || process.cwd();
            var excludeArr = void 0;
            var rcFilePath = files_utils_1.FilesUtils.findFileUp(".nycrc", cwd) || files_utils_1.FilesUtils.findFileUp(".nycrc.json", cwd);
            ;
            var packageJsonPath = files_utils_1.FilesUtils.findFileUp("package.json", cwd);
            if (rcFilePath) {
                this.logger.debug("Reading nyc exclude from '%s'", rcFilePath);
                excludeArr = this.readNycConfig(rcFilePath, false);
            }
            else if (packageJsonPath) {
                this.logger.debug("Reading nyc exclude from '%s'", packageJsonPath);
                excludeArr = this.readNycConfig(packageJsonPath, true);
            }
            return this.createExcludePatternFromNyc(excludeArr);
        }
        catch (err) {
            this.logger.warn("Error while trying to read nyc configuration. Error: ", err);
            return null;
        }
    };
    IgnoredFilesHandler.prototype.readNycConfig = function (file, isPackageJson) {
        var fileContent = fs.readFileSync(file);
        var configObject = JSON.parse(fileContent);
        var nycConfig = isPackageJson ? configObject.nyc : configObject;
        if (nycConfig) {
            return nycConfig.exclude;
        }
        return null;
    };
    IgnoredFilesHandler.prototype.createExcludePatternFromNyc = function (excludeArr) {
        if (!excludeArr) {
            this.logger.info("No files to exclude in nyc configuration");
            return null;
        }
        this.isUsingNycConfig = true;
        this.logger.debug("Found the following exclude patterns: " + JSON.stringify(excludeArr, null, "\t"));
        return excludeArr.join("\n");
    };
    IgnoredFilesHandler.prototype.denies = function (input) {
        try {
            return this.ignoredPatterns.ignores(input);
        }
        catch (e) {
            // TODO: report OBD
            this.logger.warn("Slignore - Error thrown while checking file " + input + ", Error: " + e.message);
            this.logger.debug(e);
            return false;
        }
    };
    IgnoredFilesHandler.prototype.accepts = function (input) {
        return !this.denies(input);
    };
    IgnoredFilesHandler.SCAN_IGNORE_FILE_NAME = ".slignore";
    IgnoredFilesHandler.GENERATED_IGNORE_FILE_NAME = ".slignore.generated";
    return IgnoredFilesHandler;
}());
exports.IgnoredFilesHandler = IgnoredFilesHandler;
//# sourceMappingURL=ignored-files-handler.js.map